@@ -6,8 +6,8 @@ from .framework import load_appclass, register_appclass, unregister_appclass
 from .manage import Manager
 #from .project import Project, MicroappProject
 from .project import Project
-from .group import (Group, GroupCmd, DepartureNode, ArrivalNode,
-                    EntryNode, ExitNode, Hub, AppEdge)
+from .group import (Group, GroupCmd, DepartureEntity, ArrivalEntity,
+                    EntryEntity, ExitEntity, Node, AppEdge)
 from .app import App
 
 class MicroappProject(Project):
