@@ -42,11 +42,11 @@ class Edge(object):
         return edge
 
 
-class Node(object):
+class Entity(object):
     pass
 
 
-class DynamicNode(Node):
+class DynamicEntity(Entity):
 
     @abc.abstractmethod
     def transform(self):
@@ -86,18 +86,19 @@ class AppEdge(Edge):
         return 0, fwds
 
 
-class DepartureNode(Node):
+class DepartureEntity(Entity):
 
     def __new__(cls, *vargs, **kwargs):
 
-        obj = super(DepartureNode, cls).__new__(cls)
+        obj = super(DepartureEntity, cls).__new__(cls)
         obj._nodefwd = appdict()
         obj._num_edges = 0
         obj._edges = []
 
         return obj
 
-    def update_forward(self, edge, fwd):
+    def update_edge_forward(self, edge, fwd):
+        """update edge forwards"""
 
         if edge in self._nodefwd:
             self._nodefwd[edge].update(fwd)
@@ -105,7 +106,9 @@ class DepartureNode(Node):
         else:
             self._nodefwd[edge] = appdict(fwd)
 
-    def edge_forward(self, edge):
+    def get_edge_forward(self, edge):
+        """get edge forwrads"""
+
         return appdict(self._nodefwd[edge])
 
     def add_edge(self, edge, arrivalnode):
@@ -119,13 +122,48 @@ class DepartureNode(Node):
         return list(self._edges)
 
 
-class ArrivalNode(Node):
+class ArrivalEntity(Entity):
+
+    def __new__(cls, *vargs, **kwargs):
+
+        obj = super(ArrivalEntity, cls).__new__(cls)
+
+        if "unload_method" in kwargs:
+            obj._unload_method = kwargs.pop("unload_method")
+
+        else:
+            obj._unload_method = "overwrite"
+
+        return obj
 
     def is_quorum(self, total_arrivals, present_arrivals):
 
         return set(total_arrivals) == set(a[0] for a in present_arrivals)
 
-    def node_forward(self, arrived):
+    def unload_edgedata(self, edge, edgedata, nodedata, unload_method=None):
+
+        # data => key: exit node, value: forwards
+        if unload_method is None:
+            unload_method = self._unload_method
+
+        if unload_method == "block":
+            pass
+
+        elif unload_method == "overwrite":
+            nodedata.update(edgedata)
+
+        elif unload_method == "accumulate":
+            for k, v in edgedata.items():
+                if k in nodedata:
+                    nodedata[k].append(v)
+
+                else:
+                    nodedata[k] = [v]
+        else:
+            raise UsageError("Unknown unload method: %s" % self._unload_method)
+
+    def collect_edge_forward(self, arrived):
+        """populate forwards with forwarded from each edges"""
 
         fwds = appdict()
 
@@ -133,12 +171,9 @@ class ArrivalNode(Node):
             if ret == 0:
                 for k, v in afwd.items():
                     if k in fwds:
-                        if isinstance(fwds[k], list):
-                            fwds[k].append(v)
-                        else:
-                            fwds[k] = [fwds[k], v]
+                        fwds[k].append(v)
                     else:
-                        fwds[k] = v
+                        fwds[k] = [v]
             else:
                 raise UsageError("Path '%s' returns with non-zero "
                                 "code at '%s' with forward of '%s'." % (
@@ -146,19 +181,19 @@ class ArrivalNode(Node):
         return fwds
 
 
-class EntryNode(DepartureNode):
+class EntryEntity(DepartureEntity):
     pass
 
 
-class ExitNode(ArrivalNode):
+class ExitEntity(ArrivalEntity):
     pass
 
 
-class Hub(DepartureNode, ArrivalNode):
+class Node(DepartureEntity, ArrivalEntity):
     pass
 
 
-class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
+class Group(MicroappObject, GroupArgParser, EntryEntity, ExitEntity):
 
     def __new__(cls, mgr, *vargs, **kwargs):
 
@@ -197,6 +232,10 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
                 type=bool, help="assertion test on input")
         obj.add_argument("--assert-out", metavar="expr", action="append",
                 type=bool, delay=True, help="assertion test on output")
+
+        obj.add_argument("--data-join", type=str, default="overwrite",
+                help="data combining method(block, overwrite:default, accumulate)")
+
         #obj.add_argument('--version', action='version', version=(cls._name_
         #        + " " + cls._version_))
 
@@ -207,6 +246,35 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
 
         return obj
 
+    def group_forward(self, args, data):
+        """populate forwards with forwarded from exit nodes"""
+
+        fwdtype = args.data_join["_"]
+        grp_fwds = appdict()
+
+        # data => key: exit node, value: forwards
+
+        if fwdtype == "block":
+            pass
+
+        elif fwdtype == "overwrite":
+            # discard terminal node info
+            for v in data.values():
+                grp_fwds.update(v)
+
+        elif fwdtype == "accumulate":
+            # preserve arrival node info
+            for k, v in data.items():
+                if k in grp_fwds:
+                    grp_fwds.append(v)
+
+                else:
+                    grp_fwds[k] = [v]
+        else:
+            raise UsageError("Unknown group forward type: %s" % fwdtype)
+
+        return grp_fwds
+
     def connect_edge(self, departurenode, edge, arrivalnode):
 
         departurenode.add_edge(edge, arrivalnode)
@@ -217,10 +285,10 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
         else:
             self._arrivalnodes[arrivalnode] = [edge]
 
-        if isinstance(departurenode, EntryNode):
+        if isinstance(departurenode, EntryEntity):
             self._entrynodes[departurenode] = None
 
-        if isinstance(arrivalnode, ExitNode):
+        if isinstance(arrivalnode, ExitEntity):
             self._exitnodes[arrivalnode] = None
 
 
@@ -235,15 +303,6 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
     def connect(self, args, subargs):
         pass
 
-    def group_forward(self, args, data):
-
-        grp_fwds = appdict()
-
-        for v in data.values():
-            grp_fwds.update(v)
-
-        return grp_fwds
-
 
     def _init_multiproc(self, nprocs):
 
@@ -269,7 +328,7 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
             dep, (edge, arr) = paths.pop()
 
             child_perform, args, kwargs = proxy.pack_launch(
-                                            dep.edge_forward(edge), edge, arr)
+                                            dep.get_edge_forward(edge), edge, arr)
 
             # async launch process
             self.logger.debug("edge %d runs on a new process: (%s, %s, %s)." %
@@ -354,12 +413,12 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
             # TODO: add --data argument
             for path in enode.get_departure_paths():
                 edge = path[0]
-                enode.update_forward(edge, fwds)
+                enode.update_edge_forward(edge, fwds)
 
                 if assigned_input:
                     if edge._bid < len(assigned_input[1]):
                         ainput = {assigned_input[0]: assigned_input[1][edge._bid]}
-                        enode.update_forward(edge, ainput)
+                        enode.update_edge_forward(edge, ainput)
 
                 ready_paths.append((enode, path))
 
@@ -389,7 +448,7 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
                     # pop ready path
                     departure, (edge, arrival) = ready_paths.pop(0)
 
-                    out, fwd = edge.run(self.manager, departure.edge_forward(edge))
+                    out, fwd = edge.run(self.manager, departure.get_edge_forward(edge))
 
                     just_arrived.append(arrival)
 
@@ -437,25 +496,38 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
                 # check if quorum
                 if arrival.is_quorum(self._arrivalnodes[arrival], arrived[arrival]):
 
+                    node_fwds = appdict()
+
                     # merge paths' output
-                    node_fwds = arrival.node_forward(arrived[arrival])
+                    for edge, ret, afwd in arrived[arrival]:
+                        if ret == 0:
+                            if args.data_join:
+                                arrival.unload_edgedata(edge, afwd, node_fwds,
+                                        unload_method=args.data_join["_"])
+                            else:
+                                arrival.unload_edgedata(edge, afwd, node_fwds)
+
+                        else:
+                            raise UsageError("Path '%s' returns with non-zero "
+                                            "code at '%s' with forward of '%s'." % (
+                                            str(edge), str(self), str(afwd)))
 
                     # TODO: find good usage of dynamic node
-                    if isinstance(arrival, DynamicNode):
+                    if isinstance(arrival, DynamicEntity):
                         arrival = arrival.transform()
 
-                    if isinstance(arrival, ExitNode):
+                    if isinstance(arrival, ExitEntity):
                         terminated[arrival] = node_fwds
 
                         # check if to exit
                         if self.is_finished(terminated):
                             break
 
-                    elif isinstance(arrival, DepartureNode):
+                    elif isinstance(arrival, DepartureEntity):
 
                         # add paths
                         for path in arrival.get_departure_paths():
-                            arrival.update_forward(path[0], node_fwds)
+                            arrival.update_edge_forward(path[0], node_fwds)
                             ready_paths.append((arrival, path))
 
         if PY3 and args.multiproc:
@@ -463,6 +535,7 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
 
         # merge terminals' output
         grp_fwds = self.group_forward(args, terminated)
+
         if not grp_fwds:
             grp_fwds = appdict()
 
@@ -496,12 +569,7 @@ class Group(MicroappObject, GroupArgParser, EntryNode, ExitNode):
 class GroupCmd(Group):
 
     _name_ = "group"
-    _version_ = "0.1.1"
-
-    def __init__(self, mgr):
-
-        self.add_argument("--forwarding", type=str, default="overwrite",
-                help="group forward method(pass, block, overwrite:default, accumulate)")
+    _version_ = "0.1.2"
 
     def connect(self, args, subargs):
 
@@ -525,30 +593,3 @@ class GroupCmd(Group):
         else:
             self.connect_edge(self, edge, self)
 
-    def group_forward(self, args, data):
-
-        fwdtype = args.forwarding["_"]
-        grp_fwds = appdict()
-
-        if fwdtype == "block":
-            pass
-
-        elif fwdtype == "pass":
-            grp_fwds.update(data)
-
-        elif fwdtype == "overwrite":
-            # discard arrival node info
-            for v in data.values():
-                grp_fwds.update(v)
-
-        elif fwdtype == "accumulate":
-            for k, v in data.items():
-                if k in grp_fwds:
-                    grp_fwds.append(v)
-
-                else:
-                    grp_fwds[k] = [v]
-        else:
-            raise UsageError("Unknown group forward type: %s" % fwdtype)
-
-        return grp_fwds
